@@ -8,26 +8,33 @@ from collections import namedtuple
 
 from celery import Celery
 import redis.exceptions
+import kombu.exceptions
 
 from firexapp.celery_manager import CeleryManager
 from firexapp.submit.uid import Uid
 from firexapp.broker_manager.broker_factory import BrokerFactory, REDIS_BIN_ENV
-from firexkit.inspect import get_active, get_revoked
+from firexkit.inspect import get_active, get_revoked, get_active_queues
 from firexapp.common import qualify_firex_bin, select_env_vars
 
 logger = logging.getLogger(__name__)
 
 
-CELERY_SHUTDOWN_WAIT = 5 * 60
+DEFAULT_CELERY_SHUTDOWN_TIMEOUT = 5 * 60
 MaybeCeleryActiveTasks = namedtuple('MaybeCeleryActiveTasks', ['celery_read_success', 'active_tasks'])
 
 
-def launch_background_shutdown(logs_dir, reason):
+def launch_background_shutdown(logs_dir, reason, celery_shutdown_timeout=DEFAULT_CELERY_SHUTDOWN_TIMEOUT):
     try:
-        shutdown_cmd = [qualify_firex_bin("firex_shutdown"), "--logs_dir",  logs_dir]
+        shutdown_cmd = [qualify_firex_bin("firex_shutdown"),
+                        "--logs_dir",  logs_dir,
+                        "--celery_shutdown_timeout", str(celery_shutdown_timeout)]
         if reason:
             shutdown_cmd += ['--reason', reason]
-        pid = subprocess.Popen(shutdown_cmd, close_fds=True, env=select_env_vars([REDIS_BIN_ENV, 'PATH'])).pid
+        pid = subprocess.Popen(shutdown_cmd,
+                               close_fds=True,
+                               env=select_env_vars([REDIS_BIN_ENV, 'PATH']),
+                               preexec_fn=os.setpgrp,
+                               ).pid
     except Exception as e:
         logger.error("SHUTDOWN PROCESS FAILED TO LAUNCH -- REDIS WILL LEAK.")
         logger.error(e)
@@ -58,7 +65,7 @@ def wait_for_broker_shutdown(broker, timeout=15, force_kill=True):
     return not broker.is_alive()
 
 
-def get_active_broker_safe(broker, celery_app):
+def _inspect_broker_safe(inspect_fn, broker, celery_app):
     #
     # Note: get_active can hang in celery/kombu library if broker is down.
     # Checking for broker.is_alive() is an attempt to prevent that, but note
@@ -66,11 +73,22 @@ def get_active_broker_safe(broker, celery_app):
     if not broker.is_alive():
         return None
     try:
-        return get_active(inspect_retry_timeout=4, celery_app=celery_app)
-    except redis.exceptions.ConnectionError:
+        return inspect_fn(inspect_retry_timeout=4, celery_app=celery_app)
+    except (redis.exceptions.ConnectionError, kombu.exceptions.DecodeError) as e:
+        logger.warning(e)
         return None
 
 
+def get_active_broker_safe(broker, celery_app):
+    return _inspect_broker_safe(get_active, broker, celery_app)
+
+
+def is_celery_responsive(broker, celery_app):
+    # use get_active_queues instead of get_active to check Celery responsiveness
+    # because get_active can fail to deserialize its response (i.e. raise DecodeError)
+    return _inspect_broker_safe(get_active_queues, broker, celery_app)
+
+
 def get_revoked_broker_safe(broker, celery_app):
     if not broker.is_alive():
         return None
@@ -129,6 +147,10 @@ def init():
                         required=True)
     parser.add_argument("--reason", help="A reason that will be logged for clarity.",
                         required=False, default='No reason provided.')
+    parser.add_argument("--celery_shutdown_timeout", help="Timeout in seconds for which to wait for Celery shutdown before terminating Celery processes individually."
+                        " Celery will wait for task completion after receiving a graceful shutdown, so this timeout should consider how "
+                        "long FireX shutdown should be willing to wait for remaining tasks.",
+                        default=DEFAULT_CELERY_SHUTDOWN_TIMEOUT, type=int)
     args = parser.parse_args()
     logs_dir = args.logs_dir
 
@@ -137,10 +159,10 @@ def init():
                         format='[%(asctime)s %(levelname)s] %(message)s',
                         datefmt="%Y-%m-%d %H:%M:%S")
 
-    return logs_dir, args.reason
+    return logs_dir, args.reason, args.celery_shutdown_timeout
 
 
-def shutdown_run(logs_dir, reason='No reason provided'):
+def shutdown_run(logs_dir, celery_shutdown_timeout, reason='No reason provided'):
     logger.info(f"Shutting down due to reason: {reason}")
     logger.info(f"Shutting down with logs: {logs_dir}.")
     broker = BrokerFactory.broker_manager_from_logs_dir(logs_dir)
@@ -151,15 +173,15 @@ def shutdown_run(logs_dir, reason='No reason provided'):
                         accept_content=['pickle', 'json'])
 
     try:
-        if get_active_broker_safe(broker, celery_app):
+        if is_celery_responsive(broker, celery_app):
             revoke_active_tasks(broker, celery_app)
 
             logger.info("Found active Celery; sending Celery shutdown.")
             celery_app.control.shutdown()
 
-            celery_shutdown_success = celery_manager.wait_for_shutdown(CELERY_SHUTDOWN_WAIT)
+            celery_shutdown_success = celery_manager.wait_for_shutdown(celery_shutdown_timeout)
             if not celery_shutdown_success:
-                logger.warning(f"Celery not shutdown after {CELERY_SHUTDOWN_WAIT} secs, force killing instead.")
+                logger.warning(f"Celery not shutdown after {celery_shutdown_timeout} secs, force killing instead.")
                 celery_manager.shutdown()
             else:
                 logger.debug("Confirmed Celery shutdown successfully.")
@@ -180,8 +202,8 @@ def shutdown_run(logs_dir, reason='No reason provided'):
 
 
 def main():
-    logs_dir, reason = init()
+    logs_dir, reason, celery_shutdown_timeout = init()
     try:
-        shutdown_run(logs_dir, reason)
+        shutdown_run(logs_dir, celery_shutdown_timeout, reason)
     except Exception as e:
         logger.exception(e)
