@@ -30,7 +30,7 @@ from firexapp.submit.console import setup_console_logging
 from firexapp.application import import_microservices, get_app_tasks, get_app_task
 from firexapp.engine.celery import app
 from firexapp.broker_manager.broker_factory import BrokerFactory
-from firexapp.submit.shutdown import launch_background_shutdown
+from firexapp.submit.shutdown import launch_background_shutdown, DEFAULT_CELERY_SHUTDOWN_TIMEOUT
 from firexapp.submit.install_configs import load_new_install_configs, FireXInstallConfigs, INSTALL_CONFIGS_ENV_NAME
 from firexapp.submit.arguments import whitelist_arguments
 from firexapp.common import dict2str, silent_mkdir, create_link
@@ -82,7 +82,6 @@ class AdjustCeleryConcurrency(argparse.Action):
 class SubmitBaseApp:
     SUBMISSION_LOGGING_FORMATTER = '[%(asctime)s %(levelname)s] %(message)s'
     DEFAULT_MICROSERVICE = None
-    PRIMARY_WORKER_NAME = "mc"
 
     install_configs: FireXInstallConfigs
 
@@ -162,6 +161,8 @@ class SubmitBaseApp:
                                    help='Wait for tracking services (e.g. Flame) to indicate they are ready to release '
                                         'the console before doing so.', nargs='?', const=True,
                                    default=True, action=OptionalBoolean)
+        submit_parser.add_argument('--celery_shutdown_timeout', help='How long to wait in seconds for Celery during shutdown.',
+                                   default=DEFAULT_CELERY_SHUTDOWN_TIMEOUT, type=int)
 
         submit_parser.set_defaults(func=self.run_submit)
 
@@ -404,7 +405,7 @@ class SubmitBaseApp:
         from firexapp.celery_manager import CeleryManager
         celery_manager = CeleryManager(logs_dir=self.uid.logs_dir, plugins=plugins)
         cpu_count = multiprocessing.cpu_count()
-        celery_manager.start(workername=self.PRIMARY_WORKER_NAME,
+        celery_manager.start(workername=app.conf.primary_worker_name,
                              wait=True,
                              concurrency=args.celery_concurrency,
                              autoscale=None if args.celery_concurrency else (cpu_count, cpu_count*8),
@@ -543,7 +544,7 @@ class SubmitBaseApp:
                     disable_async_result(chain_result)
 
         logger.debug("Running FireX self destruct")
-        launch_background_shutdown(self.uid.logs_dir, reason)
+        launch_background_shutdown(self.uid.logs_dir, reason, getattr(self.submit_args, 'celery_shutdown_timeout', DEFAULT_CELERY_SHUTDOWN_TIMEOUT))
 
     @classmethod
     def validate_argument_applicability(cls, chain_args, args, all_tasks):
